@@ -4,7 +4,7 @@
  *
  * Parts of this program comes from the XfKC tool:
  * Copyright (C) 2006 Gauvain Pocentek <gauvainpocentek@gmail.com>
- * 
+ *
  * A part of this file comes from the gnome keyboard capplet (control-center):
  * Copyright (C) 2003 Sergey V. Oudaltsov <svu@users.sourceforge.net>
  *
@@ -37,7 +37,7 @@
 #define xkb_cairo_arc_for_label(cr, x, y, r, a1, a2) xx = x; \
     yy = y; \
     cairo_device_to_user (cr, &xx, &yy); \
-    cairo_arc (cr, xx, yy, r, a1, a2);  
+    cairo_arc (cr, xx, yy, r, a1, a2);
 
 #define xkb_cairo_move_to(cr, x, y) xx = x; \
     yy = y; \
@@ -64,7 +64,7 @@ gint font_sizes[113] = {
 
 void
 xkb_cairo_draw_flag (cairo_t *cr,
-                     gchar *group_name,
+                     const gchar *group_name,
                      gint panel_size,
                      gint actual_width,
                      gint actual_height,
@@ -73,8 +73,6 @@ xkb_cairo_draw_flag (cairo_t *cr,
                      gint variant_markers_count,
                      GdkColor fgcolor)
 {
-    g_assert (cr != NULL);
-
     gchar *filename;
     RsvgHandle *handle;
     RsvgDimensionData dim;
@@ -83,16 +81,23 @@ xkb_cairo_draw_flag (cairo_t *cr,
     gint i;
     double layoutx, layouty;
 
+    g_assert (cr != NULL);
+
+    if (!group_name)
+        return;
+
     filename = xkb_util_get_flag_filename (group_name);
     handle = rsvg_handle_new_from_file (filename, NULL);
+    g_free (filename);
+
     if (!handle)
     {
-        xkb_cairo_draw_label (cr, group_name, 
+        xkb_cairo_draw_label (cr, group_name,
                 panel_size,
                 actual_width, actual_height,
                 width, height,
                 variant_markers_count,
-                fgcolor); 
+                fgcolor);
         return;
     }
 
@@ -138,7 +143,7 @@ xkb_cairo_draw_flag (cairo_t *cr,
 
 void
 xkb_cairo_draw_label (cairo_t *cr,
-                      gchar *group_name,
+                      const gchar *group_name,
                       gint panel_size,
                       gint actual_width,
                       gint actual_height,
@@ -147,9 +152,7 @@ xkb_cairo_draw_label (cairo_t *cr,
                       gint variant_markers_count,
                       GdkColor fgcolor)
 {
-    g_assert (cr != NULL);
-
-    gchar **normalized_group_name;
+    gchar *normalized_group_name;
     gchar font_str[80];
     gint pango_width, pango_height;
     gint layoutx, layouty;
@@ -157,11 +160,19 @@ xkb_cairo_draw_label (cairo_t *cr,
     gint i;
     gint radius;
 
-    normalized_group_name = xkb_util_normalize_group_name (group_name);
-
     PangoLayout *layout;
     PangoFontDescription *desc;
+
+    g_assert (cr != NULL);
+
     layout = pango_cairo_create_layout (cr);
+    normalized_group_name = xkb_util_normalize_group_name (group_name);
+
+    if (!g_utf8_validate (normalized_group_name, -1, NULL))
+    {
+        g_object_unref (layout);
+        g_free (normalized_group_name);
+    }
 
     pango_layout_set_text (layout, normalized_group_name, -1);
     g_sprintf (font_str, XKB_PREFERRED_FONT, font_sizes[panel_size - 16]);
@@ -183,9 +194,9 @@ xkb_cairo_draw_label (cairo_t *cr,
         cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
         cairo_set_line_width (cr, 1);
         radius = (panel_size < 32) ? 1.5 : 2.5;
-        xkb_cairo_arc_for_label (cr, 
-                layoutx + pango_width + 3 + (7 * i), 
-                layouty + pango_height - (pango_height / 5), 
+        xkb_cairo_arc_for_label (cr,
+                layoutx + pango_width + 3 + (7 * i),
+                layouty + pango_height - (pango_height / 5),
                 radius, 0, 2 * G_PI
         );
         cairo_fill (cr);
