@@ -4,7 +4,7 @@
  *
  * Parts of this program comes from the XfKC tool:
  * Copyright (C) 2006 Gauvain Pocentek <gauvainpocentek@gmail.com>
- * 
+ *
  * A part of this file comes from the gnome keyboard capplet (control-center):
  * Copyright (C) 2003 Sergey V. Oudaltsov <svu@users.sourceforge.net>
  *
@@ -39,6 +39,7 @@
 #include "xkb-settings-dialog.h"
 #include "xkb-util.h"
 
+GtkTreeIter current_iter;
 GtkWidget *settings_dialog;
 GtkWidget *default_layout_menu;
 
@@ -64,12 +65,12 @@ enum enumeration
     NUM
 };
 
-gchar       *xkb_settings_layout_dialog_run     ();
-void         xkb_settings_update_from_ui        (t_xkb *xkb);
+static gchar       *xkb_settings_layout_dialog_run     ();
+static void         xkb_settings_update_from_ui        (t_xkb *xkb);
 
 /**************************************************************/
 
-void
+static void
 on_settings_close (GtkDialog *dialog, gint response, t_xkb *xkb)
 {
     xfce_panel_plugin_unblock_menu (xkb->plugin);
@@ -81,21 +82,21 @@ on_settings_close (GtkDialog *dialog, gint response, t_xkb *xkb)
     gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-void 
-on_display_type_changed (GtkComboBox *cb, t_xkb *xkb) 
+static void
+on_display_type_changed (GtkComboBox *cb, t_xkb *xkb)
 {
     xkb->display_type = gtk_combo_box_get_active (cb);
     xkb_refresh_gui (xkb);
 }
 
-void 
-on_group_policy_changed (GtkComboBox *cb, t_xkb *xkb) 
+static void
+on_group_policy_changed (GtkComboBox *cb, t_xkb *xkb)
 {
     xkb->settings->group_policy = gtk_combo_box_get_active (cb);
 }
 
 /* from the gnome control center keyboard applet */
-char *
+static char *
 xci_desc_to_utf8 (XklConfigItem * ci)
 {
     char *sd = g_strstrip (ci->description);
@@ -103,12 +104,13 @@ xci_desc_to_utf8 (XklConfigItem * ci)
 }
 /**/
 
-void
+static void
 xkb_settings_fill_layout_tree_model_with_config (t_xkb *xkb)
 {
+    GtkTreeIter iter;
     gint layout_nb = 0;
 
-    t_xkb_kbd_config *config = xkb->settings->kbd_config;   
+    t_xkb_kbd_config *config = xkb->settings->kbd_config;
 
     char **layouts = g_strsplit_set (config->layouts, ",", 0);
     char **variants = g_strsplit_set (config->variants, ",", 0);
@@ -123,6 +125,9 @@ xkb_settings_fill_layout_tree_model_with_config (t_xkb *xkb)
                             -1);
         layout_nb += 1;
     }
+
+    g_strfreev (layouts);
+    g_strfreev (variants);
 }
 
 static void
@@ -130,6 +135,9 @@ xkb_settings_add_toggle_options_to_list (XklConfigRegistry * config_registry,
                                          XklConfigItem * config_item,
                                          t_xkb *xkb)
 {
+    GtkTreeIter iter;
+    char *utf_option_name;
+
     /* add a possibility to set no toggle layout combination */
     if (config_item == NULL)
     {
@@ -140,9 +148,9 @@ xkb_settings_add_toggle_options_to_list (XklConfigRegistry * config_registry,
         return;
     }
 
-    char *utf_option_name = xci_desc_to_utf8 (config_item);
+    utf_option_name = xci_desc_to_utf8 (config_item);
     gtk_list_store_append (xkb->toggle_options_store, &iter);
-    gtk_list_store_set (xkb->toggle_options_store, &iter, 
+    gtk_list_store_set (xkb->toggle_options_store, &iter,
                       DESC, utf_option_name,
                       NOM, config_item->name, -1);
     g_free (utf_option_name);
@@ -153,6 +161,9 @@ xkb_settings_add_compose_key_position_options_to_list (XklConfigRegistry * confi
                                                      XklConfigItem * config_item,
                                                      t_xkb *xkb)
 {
+    GtkTreeIter iter;
+    char *utf_option_name;
+
     /* add a possibility to set no position for the compose key */
     if (config_item == NULL)
     {
@@ -163,7 +174,7 @@ xkb_settings_add_compose_key_position_options_to_list (XklConfigRegistry * confi
         return;
     }
 
-    char *utf_option_name = xci_desc_to_utf8 (config_item);
+    utf_option_name = xci_desc_to_utf8 (config_item);
     gtk_list_store_append (xkb->compose_key_options_store, &iter);
     gtk_list_store_set (xkb->compose_key_options_store, &iter,
                         DESC, utf_option_name,
@@ -176,32 +187,38 @@ xkb_settings_add_kbd_model_to_list (XklConfigRegistry * config_registry,
                                     XklConfigItem * config_item,
                                     t_xkb *xkb)
 {
+    GtkTreeIter iter;
     char *utf_model_name = xci_desc_to_utf8 (config_item);
     gtk_list_store_append (xkb->combo_store, &iter);
-    gtk_list_store_set (xkb->combo_store, &iter, 
+    gtk_list_store_set (xkb->combo_store, &iter,
                       DESC, utf_model_name,
                       NOM, config_item->name, -1);
     g_free (utf_model_name);
 }
 
-void
+static void
 xkb_settings_set_toggle_option_combo_default_value (t_xkb *xkb)
 {
+    GtkTreeModel *model;
+    GtkTreeIter iter;
     gchar *id;
 
     t_xkb_kbd_config *config = xkb->settings->kbd_config;
 
     model = GTK_TREE_MODEL (xkb->toggle_options_store);
-    gtk_tree_model_get_iter_first (model, &iter);
+    if (!gtk_tree_model_get_iter_first (model, &iter))
+        return;
+
     gtk_tree_model_get (model, &iter, NOM, &id, -1);
 
     if (config->toggle_option == NULL)
     {
         gtk_combo_box_set_active_iter (GTK_COMBO_BOX (xkb->toggle_options_combo), &iter);
+        g_free (id);
         return;
     }
 
-    if (strcmp (id, config->toggle_option) == 0 )
+    if (strcmp (id, config->toggle_option) == 0)
     {
         gtk_combo_box_set_active_iter (GTK_COMBO_BOX (xkb->toggle_options_combo), &iter);
     }
@@ -209,6 +226,7 @@ xkb_settings_set_toggle_option_combo_default_value (t_xkb *xkb)
     {
         while (gtk_tree_model_iter_next(model, &iter))
         {
+            g_free (id);
             gtk_tree_model_get (model, &iter, NOM, &id, -1);
 
             if (strcmp (id, config->toggle_option) == 0)
@@ -218,24 +236,28 @@ xkb_settings_set_toggle_option_combo_default_value (t_xkb *xkb)
             }
         }
     }
-    
+
     g_free (id);
 }
 
-void
+static void
 xkb_settings_set_compose_key_position_combo_default_value (t_xkb *xkb)
 {
+    GtkTreeModel *model;
+    GtkTreeIter iter;
     gchar *id;
 
     t_xkb_kbd_config *config = xkb->settings->kbd_config;
 
     model = GTK_TREE_MODEL (xkb->compose_key_options_store);
-    gtk_tree_model_get_iter_first (model, &iter);
+    if (!gtk_tree_model_get_iter_first (model, &iter))
+        return;
     gtk_tree_model_get (model, &iter, NOM, &id, -1);
 
     if (config->compose_key_position == NULL)
     {
         /* select the empty option if compose_key_position is null */
+        g_free (id);
         gtk_combo_box_set_active_iter (GTK_COMBO_BOX (xkb->compose_key_options_combo), &iter);
         return;
     }
@@ -248,6 +270,7 @@ xkb_settings_set_compose_key_position_combo_default_value (t_xkb *xkb)
     {
         while (gtk_tree_model_iter_next (model, &iter))
         {
+            g_free (id);
             gtk_tree_model_get (model, &iter, NOM, &id, -1);
 
             if (strcmp (id, config->compose_key_position) == 0)
@@ -261,14 +284,18 @@ xkb_settings_set_compose_key_position_combo_default_value (t_xkb *xkb)
     g_free (id);
 }
 
-void
+static void
 xkb_settings_set_kbd_combo_default_value (t_xkb *xkb)
 {
+    GtkTreeModel *model;
+    GtkTreeIter iter;
     gchar *id;
     t_xkb_kbd_config *config = xkb->settings->kbd_config;
 
     model = GTK_TREE_MODEL (xkb->combo_store);
-    gtk_tree_model_get_iter_first (model, &iter);
+    if (!gtk_tree_model_get_iter_first (model, &iter))
+        return;
+
     gtk_tree_model_get (model, &iter, NOM, &id, -1);
     if (strcmp (id, config->model) == 0 )
         gtk_combo_box_set_active_iter (GTK_COMBO_BOX (xkb->kbd_model_combo), &iter);
@@ -276,6 +303,7 @@ xkb_settings_set_kbd_combo_default_value (t_xkb *xkb)
     {
         while (gtk_tree_model_iter_next(model, &iter))
         {
+            g_free (id);
             gtk_tree_model_get (model, &iter, NOM, &id, -1);
 
             if (strcmp (id, config->model) == 0)
@@ -285,17 +313,20 @@ xkb_settings_set_kbd_combo_default_value (t_xkb *xkb)
             }
         }
     }
-    
+
     g_free (id);
 }
 
 static gint
 xkb_settings_get_group_count (t_xkb *xkb)
 {
+    GtkTreeModel *model;
+    GtkTreeIter iter;
     gint count = 1;
 
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (xkb->layout_tree_view));
-    gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter); 
+    if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter))
+        return 0;
     while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter))
         count++;
     return count;
@@ -306,9 +337,9 @@ xkb_settings_show_hide_layout_buttons (t_xkb *xkb)
 {
     gint nb = xkb_settings_get_group_count (xkb);
     gint max_nb = xkb_config_get_max_layout_number ();
-    gtk_widget_set_sensitive (xkb->add_layout_btn, 
+    gtk_widget_set_sensitive (xkb->add_layout_btn,
             (nb < max_nb && !xkb->settings->never_modify_config));
-    gtk_widget_set_sensitive (xkb->rm_layout_btn, 
+    gtk_widget_set_sensitive (xkb->rm_layout_btn,
             (nb > 1 && !xkb->settings->never_modify_config));
 }
 
@@ -319,21 +350,28 @@ xkb_settings_edit_layout_btn_show (GtkTreeView *tree_view,
     GtkTreePath *p;
     GtkTreeViewColumn *c;
     gtk_tree_view_get_cursor (GTK_TREE_VIEW (tree_view), &p, &c);
-    gtk_widget_set_sensitive (xkb->edit_layout_btn, 
+    gtk_widget_set_sensitive (xkb->edit_layout_btn,
             (p != NULL && !xkb->settings->never_modify_config));
+    gtk_tree_path_free (p);
 }
 
 static void
 xkb_settings_edit_layout (GtkWidget *widget, t_xkb *xkb)
 {
+    GtkTreeModel *model;
     gchar *c;
     gboolean is_default;
 
+    model = gtk_tree_view_get_model (GTK_TREE_VIEW (xkb->layout_tree_view));
+
     c = xkb_settings_layout_dialog_run ();
     if (c != NULL)
     {
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (xkb->layout_tree_view));
+        GtkTreeSelection *selection;
+        GtkTreeIter iter;
         gchar **strings;
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (xkb->layout_tree_view));
         strings = g_strsplit_set(c, ",", 0);
 
         gtk_tree_selection_get_selected (selection, &model, &iter);
@@ -344,11 +382,10 @@ xkb_settings_edit_layout (GtkWidget *widget, t_xkb *xkb)
                             VARIANTS, strings[1],
                             -1);
         xkb_settings_update_from_ui (xkb);
-        g_free(strings);
+        g_strfreev (strings);
     }
     g_free(c);
     xkb_settings_edit_layout_btn_show (GTK_TREE_VIEW (xkb->layout_tree_view), xkb);
-    
 }
 
 static void
@@ -358,7 +395,9 @@ xkb_settings_add_layout (GtkWidget *widget, t_xkb *xkb)
     c = xkb_settings_layout_dialog_run();
     if (c != NULL)
     {
+        GtkTreeIter iter;
         gchar **strings;
+
         strings = g_strsplit_set(c, ",", 0);
         gtk_list_store_append (xkb->layout_store, &iter);
         gtk_list_store_set (xkb->layout_store, &iter,
@@ -368,17 +407,21 @@ xkb_settings_add_layout (GtkWidget *widget, t_xkb *xkb)
                         -1);
         xkb_settings_show_hide_layout_buttons (xkb);
         xkb_settings_update_from_ui (xkb);
-        g_free (strings);
+        g_strfreev (strings);
     }
     g_free (c);
     xkb_settings_edit_layout_btn_show (GTK_TREE_VIEW (xkb->layout_tree_view), xkb);
 }
-    
+
 static void
 xkb_settings_rm_layout (GtkWidget *widget, t_xkb *xkb)
 {
+    GtkTreeSelection *selection;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
     gboolean is_default;
 
+    model = gtk_tree_view_get_model (GTK_TREE_VIEW (xkb->layout_tree_view));
     selection = gtk_tree_view_get_selection (
                     GTK_TREE_VIEW (xkb->layout_tree_view));
     if (gtk_tree_selection_get_selected (selection, &model, &iter))
@@ -406,23 +449,27 @@ xkb_settings_default_layout_toggled (GtkCellRendererToggle *renderer,
 {
     /* warning, super dumb code - set all layout toggle values to
        false, then set the toggled one to true */
+    GtkTreeModel *model;
+    GtkTreeIter iter;
 
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (xkb->layout_tree_view));
-    gtk_tree_model_get_iter_first (model, &iter);
-    do
+    if (gtk_tree_model_get_iter_first (model, &iter))
     {
-        gtk_list_store_set (GTK_LIST_STORE (model), &iter, DEFAULT_LAYOUT, FALSE, -1);
-    } while (gtk_tree_model_iter_next (model, &iter));
+        do
+        {
+            gtk_list_store_set (GTK_LIST_STORE (model), &iter, DEFAULT_LAYOUT, FALSE, -1);
+        } while (gtk_tree_model_iter_next (model, &iter));
 
 
-    if (gtk_tree_model_get_iter_from_string (model, &iter, path))
-    {
-        gtk_list_store_set (GTK_LIST_STORE (model), &iter, DEFAULT_LAYOUT, TRUE, -1);
+        if (gtk_tree_model_get_iter_from_string (model, &iter, path))
+        {
+            gtk_list_store_set (GTK_LIST_STORE (model), &iter, DEFAULT_LAYOUT, TRUE, -1);
+        }
+        xkb_settings_update_from_ui (xkb);
     }
-    xkb_settings_update_from_ui (xkb);
 }
 
-gboolean
+static gboolean
 xkb_settings_config_modification_disabled_tooltip (GtkWidget *widget,
                                                    gint x, gint y,
                                                    gboolean keyboard_mode,
@@ -448,13 +495,12 @@ xfce_xkb_configure (XfcePanelPlugin *plugin,
 {
     GtkWidget *display_type_optmenu, *group_policy_combo;
     GtkWidget *vbox, *display_type_frame, *group_policy_frame, *bin;
-          
 
     GtkCellRenderer *renderer, *renderer2;
     GtkWidget *vbox1, *vbox2, *hbox, *frame;
     XklConfigRegistry *registry;
 
-    xfce_panel_plugin_block_menu (plugin); 
+    xfce_panel_plugin_block_menu (plugin);
 
     registry = xkb_config_get_xkl_registry ();
 
@@ -475,7 +521,7 @@ xfce_xkb_configure (XfcePanelPlugin *plugin,
     xkb->combo_store = gtk_list_store_new (COMBO_NUM, G_TYPE_STRING, G_TYPE_STRING);
     xkb->toggle_options_store = gtk_list_store_new (COMBO_NUM, G_TYPE_STRING, G_TYPE_STRING);
     xkb->compose_key_options_store = gtk_list_store_new (COMBO_NUM, G_TYPE_STRING, G_TYPE_STRING);
-    
+
     vbox1 = gtk_vbox_new (FALSE, 5);
     gtk_container_add (GTK_CONTAINER (vbox), vbox1);
     gtk_widget_show (vbox1);
@@ -490,7 +536,7 @@ xfce_xkb_configure (XfcePanelPlugin *plugin,
     gtk_container_add (GTK_CONTAINER (bin), xkb->kbd_model_combo);
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (xkb->kbd_model_combo), renderer, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (xkb->kbd_model_combo), renderer, "text", 0);
-    
+
     xkl_config_registry_foreach_model (registry,
                        (ConfigItemProcessFunc) xkb_settings_add_kbd_model_to_list,
                            xkb);
@@ -503,7 +549,7 @@ xfce_xkb_configure (XfcePanelPlugin *plugin,
     gtk_widget_set_sensitive (GTK_WIDGET (xkb->kbd_model_combo), !xkb->settings->never_modify_config);
     g_object_set (G_OBJECT (xkb->kbd_model_combo), "has-tooltip", TRUE, NULL);
     g_signal_connect (xkb->kbd_model_combo, "query-tooltip", G_CALLBACK (xkb_settings_config_modification_disabled_tooltip), xkb);
-    
+
     /* toggle layout option */
     frame = xfce_gtk_frame_box_new (_("Change layout option:"), &bin);
     gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
@@ -556,47 +602,47 @@ xfce_xkb_configure (XfcePanelPlugin *plugin,
     g_object_set (G_OBJECT (xkb->compose_key_options_combo), "has-tooltip", TRUE, NULL);
     g_signal_connect (xkb->compose_key_options_combo, "query-tooltip", G_CALLBACK (xkb_settings_config_modification_disabled_tooltip), xkb);
 
-    
+
     /* the actual layouts */
     frame = xfce_gtk_frame_box_new (_("Keyboard layouts:"), &bin);
     gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
     gtk_widget_show (frame);
     gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
-    
+
     hbox = gtk_hbox_new (FALSE, 5);
     gtk_container_add (GTK_CONTAINER (bin), hbox);
     gtk_widget_show (hbox);
-    
+
     // TreeView
     xkb->layout_tree_view = gtk_tree_view_new ();
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (xkb->layout_tree_view),
-                                               -1,      
-                                               "Default",  
+                                               -1,
+                                               _("Default"),
                                                renderer2,
                                                "active", DEFAULT_LAYOUT,
                                                NULL);
-                                               
+
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (xkb->layout_tree_view),
-                                               -1,      
-                                               "Layout",  
+                                               -1,
+                                               _("Layout"),
                                                renderer,
                                                "text", LAYOUTS,
                                                NULL);
-                                               
+
     renderer = gtk_cell_renderer_text_new ();
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (xkb->layout_tree_view),
-                                               -1,      
-                                               "Variant",  
+                                               -1,
+                                               _("Variant"),
                                                renderer,
                                                "text", VARIANTS,
                                                NULL);
-    
+
     xkb->layout_store = gtk_list_store_new (TREE_NUM, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING);
     xkb_settings_fill_layout_tree_model_with_config (xkb);
-    
+
     gtk_tree_view_set_model (GTK_TREE_VIEW (xkb->layout_tree_view),
                             GTK_TREE_MODEL (xkb->layout_store));
-    
+
     gtk_box_pack_start (GTK_BOX (hbox), xkb->layout_tree_view, TRUE, TRUE, 0);
     gtk_widget_set_size_request (xkb->layout_tree_view, -1, 112);
     gtk_widget_show (xkb->layout_tree_view);
@@ -671,6 +717,8 @@ xfce_xkb_configure (XfcePanelPlugin *plugin,
 
     g_signal_connect (renderer2, "toggled", G_CALLBACK (xkb_settings_default_layout_toggled), xkb);
 
+    g_object_unref (registry);
+
     gtk_widget_show (settings_dialog);
 }
 
@@ -679,69 +727,72 @@ xfce_xkb_about (XfcePanelPlugin *plugin)
 {
     GtkWidget *about;
     const gchar* authors[3] = {
-        "Alexander Iliev <sasoiliev@mamul.org>", 
-        "Gauvain Pocentek <gauvainpocentek@gmail.com>", 
+        "Alexander Iliev <sasoiliev@mamul.org>",
+        "Gauvain Pocentek <gauvainpocentek@gmail.com>",
         NULL
     };
 
     about = gtk_about_dialog_new ();
-    gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about), 
+    gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about),
             _("Keyboard Layouts Plugin"));
-    gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (about), 
+    gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (about),
             NULL);
-    gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about), 
+    gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about),
             (const gchar**) authors);
-    gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about), 
+    gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about),
             _("Allows you to configure and use multiple keyboard layouts."));
-    gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (about), 
+    gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (about),
             "http://goodies.xfce.org/");
-    gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG (about), 
+    gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG (about),
             _("Other plugins available here"));
     gtk_dialog_run (GTK_DIALOG (about));
-    gtk_widget_destroy (about); 
+    gtk_widget_destroy (about);
 }
 
 static void
 xkb_settings_add_variant_to_available_layouts_tree (XklConfigRegistry * config_registry,
-                                                    XklConfigItem * config_item, 
+                                                    XklConfigItem * config_item,
                                                     GtkTreeStore *treestore)
 {
+  GtkTreeIter child;
   char *utf_variant_name = xci_desc_to_utf8 (config_item);
 
-  gtk_tree_store_append (treestore, &child, &iter);
-  gtk_tree_store_set (treestore, &child, 
-              AVAIL_LAYOUT_TREE_COL_DESCRIPTION, utf_variant_name, 
+  gtk_tree_store_append (treestore, &child, &current_iter);
+  gtk_tree_store_set (treestore, &child,
+              AVAIL_LAYOUT_TREE_COL_DESCRIPTION, utf_variant_name,
               AVAIL_LAYOUT_TREE_COL_ID, config_item->name, -1);
   g_free (utf_variant_name);
 }
 
 static void
 xkb_settings_add_layout_to_available_layouts_tree (XklConfigRegistry * config_registry,
-                                                   XklConfigItem * config_item, 
+                                                   XklConfigItem * config_item,
                                                    GtkTreeStore *treestore)
 {
   char *utf_layout_name = xci_desc_to_utf8 (config_item);
 
-  gtk_tree_store_append (treestore, &iter, NULL);
-  gtk_tree_store_set (treestore, &iter, 
-              AVAIL_LAYOUT_TREE_COL_DESCRIPTION, utf_layout_name, 
+  gtk_tree_store_append (treestore, &current_iter, NULL);
+  gtk_tree_store_set (treestore, &current_iter,
+              AVAIL_LAYOUT_TREE_COL_DESCRIPTION, utf_layout_name,
               AVAIL_LAYOUT_TREE_COL_ID, config_item->name, -1);
   g_free (utf_layout_name);
 
   xkl_config_registry_foreach_layout_variant (config_registry, config_item->name,
-                   (ConfigItemProcessFunc)xkb_settings_add_variant_to_available_layouts_tree, 
+                   (ConfigItemProcessFunc)xkb_settings_add_variant_to_available_layouts_tree,
                                treestore);
 }
 
-gchar *
-xkb_settings_layout_dialog_run ()
+static gchar *
+xkb_settings_layout_dialog_run (void)
 {
     GtkWidget *dialog;
     GtkTreeStore *treestore;
     GtkWidget *tree_view = gtk_tree_view_new ();
     GtkCellRenderer *renderer;
+    GtkTreeViewColumn *column;
     GtkWidget *scrolledw;
     XklConfigRegistry *registry;
+    int response;
 
     registry = xkb_config_get_xkl_registry ();
 
@@ -757,17 +808,19 @@ xkb_settings_layout_dialog_run ()
     gtk_window_set_icon_name (GTK_WINDOW (dialog), "xfce4-keyboard");
 
     treestore = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
-    
+
     xkl_config_registry_foreach_layout (registry, (ConfigItemProcessFunc)
             xkb_settings_add_layout_to_available_layouts_tree, treestore);
+    g_object_unref (registry);
 
     renderer = gtk_cell_renderer_text_new ();
-    
-    GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes (NULL,
-                                    renderer,
-                                    "text",
-                                    AVAIL_LAYOUT_TREE_COL_DESCRIPTION,
-                                    NULL);
+
+    column =
+        gtk_tree_view_column_new_with_attributes (NULL,
+                                                  renderer,
+                                                  "text",
+                                                  AVAIL_LAYOUT_TREE_COL_DESCRIPTION,
+                                                  NULL);
     gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view),
                GTK_TREE_MODEL (treestore));
     gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
@@ -780,53 +833,81 @@ xkb_settings_layout_dialog_run ()
                                     GTK_POLICY_AUTOMATIC,
                                     GTK_POLICY_AUTOMATIC);
     gtk_widget_show (scrolledw);
-    
+
     gtk_container_add (GTK_CONTAINER (scrolledw), tree_view);
     gtk_widget_show (GTK_WIDGET (tree_view));
-    
+
     gtk_window_set_default_size(GTK_WINDOW (dialog), 400, 400);
     gtk_widget_show (dialog);
 
-    int response = gtk_dialog_run (GTK_DIALOG (dialog));
-    
+    response = gtk_dialog_run (GTK_DIALOG (dialog));
+
     if (response == GTK_RESPONSE_OK)
     {
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+        GtkTreeSelection *selection;
+        GtkTreeModel *model;
+        GtkTreePath *tree_path;
+        GtkTreeIter iter;
         gchar *id;
-        gchar *strings[2];
-                
-        gtk_tree_selection_get_selected (selection, &model, &iter);
-        gtk_tree_model_get (model, &iter, AVAIL_LAYOUT_TREE_COL_ID, &id, -1);
-        
-        path = gtk_tree_model_get_path (model, &iter);
-        if (gtk_tree_path_get_depth (path) == 1)
+        gchar *result;
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+
+        if (!gtk_tree_selection_get_selected (selection, &model, &iter))
         {
-            strings[0] = id;
-            strings[1] = "";
+            gtk_widget_destroy (dialog);
+            return NULL;
         }
+        gtk_tree_model_get (model, &iter, AVAIL_LAYOUT_TREE_COL_ID, &id, -1);
+
+        tree_path = gtk_tree_model_get_path (model, &iter);
+        if (gtk_tree_path_get_depth (tree_path) == 1)
+            result = g_strconcat(id, ",", NULL);
         else
         {
-            strings[1] = id;
-            gtk_tree_path_up(path);
-            gtk_tree_model_get_iter(model, &iter, path);
-            gtk_tree_model_get (model, &iter, AVAIL_LAYOUT_TREE_COL_ID, &id, -1);
-            strings[0] = id;
+            gchar *tmp;
+
+            if (!gtk_tree_path_up(tree_path))
+            {
+                g_free (id);
+                gtk_tree_path_free (tree_path);
+                gtk_widget_destroy (dialog);
+                return NULL;
+            }
+
+            if (!gtk_tree_model_get_iter(model, &iter, tree_path))
+            {
+                g_free (id);
+                gtk_tree_path_free (tree_path);
+                gtk_widget_destroy (dialog);
+                return NULL;
+            }
+
+            gtk_tree_model_get (model, &iter, AVAIL_LAYOUT_TREE_COL_ID, &tmp, -1);
+            result = g_strconcat (tmp, ",", id, NULL);
+            g_free (tmp);
         }
 
         gtk_widget_destroy (dialog);
-        return g_strconcat(strings[0], ",", strings[1], NULL);
-        
+        g_free (id);
+        gtk_tree_path_free (tree_path);
+
+        return result;
     }
     gtk_widget_destroy (dialog);
+
     return NULL;
 }
 
-void
+static void
 xkb_settings_update_from_ui (t_xkb *xkb)
 {
-    gchar *layouts, *variants, *kbdmodel, *toggle_option, 
-          *compose_key_position, *tmp;
+    GtkTreeModel *model;
+    gchar *layouts, *variants, *kbdmodel, *toggle_option,
+          *compose_key_position;
     t_xkb_kbd_config *kbd_config = xkb->settings->kbd_config;
+    GtkTreeIter iter;
     gboolean is_default;
     gint i = 0;
 
@@ -834,6 +915,7 @@ xkb_settings_update_from_ui (t_xkb *xkb)
     if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (xkb->kbd_model_combo), &iter))
     {
         gtk_tree_model_get (model, &iter, NOM, &kbdmodel, -1);
+        g_free (kbd_config->model);
         kbd_config->model = kbdmodel;
     }
 
@@ -845,6 +927,8 @@ xkb_settings_update_from_ui (t_xkb *xkb)
         if (strcmp ("", toggle_option) == 0)
             kbd_config->toggle_option = NULL;
         else kbd_config->toggle_option = g_strdup (toggle_option);
+
+        g_free (toggle_option);
     }
 
     model = GTK_TREE_MODEL (xkb->compose_key_options_store);
@@ -856,31 +940,49 @@ xkb_settings_update_from_ui (t_xkb *xkb)
             kbd_config->compose_key_position = NULL;
         else kbd_config->compose_key_position = g_strdup (compose_key_position);
 
+        g_free (compose_key_position);
     }
 
     model = GTK_TREE_MODEL (xkb->layout_store);
-    gtk_tree_model_get_iter_first (model, &iter);
-    gtk_tree_model_get (model, &iter, DEFAULT_LAYOUT, &is_default, LAYOUTS, &layouts, VARIANTS, &variants, -1);
-    if (is_default) xkb->settings->default_group = i;
-    kbd_config->layouts = layouts;
-    if (variants != NULL)
-        kbd_config->variants = variants;
-    else
-        kbd_config->variants = "";
-    
-    i = 1;
-    while (gtk_tree_model_iter_next (model, &iter))
+    if (gtk_tree_model_get_iter_first (model, &iter))
     {
         gtk_tree_model_get (model, &iter, DEFAULT_LAYOUT, &is_default, LAYOUTS, &layouts, VARIANTS, &variants, -1);
         if (is_default) xkb->settings->default_group = i;
-        i++;
 
-        kbd_config->layouts = g_strconcat(kbd_config->layouts, ",", layouts, NULL);
+        g_free (kbd_config->layouts);
+        kbd_config->layouts = layouts;
+
+        g_free (kbd_config->variants);
         if (variants != NULL)
-            kbd_config->variants = g_strconcat(kbd_config->variants, ",", variants, NULL);
+            kbd_config->variants = variants;
         else
-            kbd_config->variants = g_strconcat(kbd_config->variants, ",", NULL);
+            kbd_config->variants = g_strdup ("");
+
+        i = 1;
+        while (gtk_tree_model_iter_next (model, &iter))
+        {
+            gchar *tmp;
+
+            gtk_tree_model_get (model, &iter, DEFAULT_LAYOUT, &is_default, LAYOUTS, &layouts, VARIANTS, &variants, -1);
+            if (is_default) xkb->settings->default_group = i;
+            i++;
+
+            tmp = g_strconcat(kbd_config->layouts, ",", layouts, NULL);
+            g_free (kbd_config->layouts);
+            kbd_config->layouts = tmp;
+            g_free (layouts);
+
+            if (variants != NULL)
+                tmp = g_strconcat(kbd_config->variants, ",", variants, NULL);
+            else
+                tmp = g_strconcat(kbd_config->variants, ",", NULL);
+
+            g_free (kbd_config->variants);
+            kbd_config->variants = tmp;
+            g_free (variants);
+        }
     }
+
     xkb_config_update_settings (xkb->settings);
     xkb_refresh_gui (xkb);
 }
